import 'package:flutter/material.dart';
import 'package:nav/enum/enum_nav_ani.dart';
import 'package:nav/nav.dart';

mixin NavDialog {
  BuildContext get context;
  NavAni ani = NavAni.Fade;

  Future<T> show<T>() async {
    switch (ani) {
      case NavAni.Left:
      case NavAni.Right:
      case NavAni.Top:
      case NavAni.Bottom:
        return showDialogWith(ani, context: context, builder: (context) {
          return dialogWidget;
        });
      case NavAni.Blink:
        return showDialogWith(ani, context: context, builder: (context) {
          return dialogWidget;
        }, durationMs: 0);
        break;
      case NavAni.Fade:
        return showDialog(
          context: context,
          builder: (context) {
            return dialogWidget;
          },
        );
        break;
      case NavAni.Ripple:
        break;
        break;
    }
  }

  Widget get dialogWidget;

  void hide() {
    Nav.pop(context);
  }
}

Widget _buildFromFadeTransition(
    BuildContext context,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child) {
  return FadeTransition(
    opacity: Tween<double>(begin: 0.0, end: 1.0).animate(animation),
    child: child,
  );
}

Widget _buildFromTopTransition(
    BuildContext context,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child) {
  return SlideTransition(
    position: Tween<Offset>(
      begin: const Offset(0.0, -1.0),
      end: Offset.zero,
    ).chain(CurveTween(curve: Cubic(0.4, 0, 0.2, 1))).animate(animation),
    child: child,
  );
}

Widget _buildFromBottomTransition(
    BuildContext context,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child) {
  return SlideTransition(
    position: Tween<Offset>(
      begin: const Offset(0.0, 1.0),
      end: Offset.zero,
    ).chain(CurveTween(curve: Cubic(0.4, 0, 0.2, 1))).animate(animation),
    child: child,
  );
}

Widget _buildFromRightTransition(
    BuildContext context,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child) {
  return SlideTransition(
    position: Tween<Offset>(
      begin: const Offset(1.0, 0.0),
      end: Offset.zero,
    ).chain(CurveTween(curve: Cubic(0.4, 0, 0.2, 1))).animate(animation),
    child: child,
  );
}

Widget _buildFromLeftTransition(
    BuildContext context,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child) {
  return SlideTransition(
    position: Tween<Offset>(
      begin: const Offset(-1.0, 0.0),
      end: Offset.zero,
    ).chain(CurveTween(curve: Cubic(0.4, 0, 0.2, 1))).animate(animation),
    child: child,
  );
}

Future<T> showDialogWith<T>(
  NavAni ani, {
  @required
      BuildContext context,
  bool barrierDismissible = true,
  @Deprecated(
      'Instead of using the "child" argument, return the child from a closure '
      'provided to the "builder" argument. This will ensure that the BuildContext '
      'is appropriate for widgets built in the dialog. '
      'This feature was deprecated after v0.2.3.')
      Widget child,
  WidgetBuilder builder,
  bool useRootNavigator = true,
  int durationMs = 500,
}) {
  assert(child == null || builder == null);
  assert(useRootNavigator != null);
  assert(debugCheckHasMaterialLocalizations(context));

  final ThemeData theme = Theme.of(context, shadowThemeOnly: true);
  return showGeneralDialog(
    context: context,
    pageBuilder: (BuildContext buildContext, Animation<double> animation,
        Animation<double> secondaryAnimation) {
      final Widget pageChild = child ?? Builder(builder: builder);
      return Builder(builder: (BuildContext context) {
        return theme != null ? Theme(data: theme, child: pageChild) : pageChild;
      });
    },
    barrierDismissible: barrierDismissible,
    barrierLabel: MaterialLocalizations.of(context).modalBarrierDismissLabel,
    barrierColor: Colors.black54,
    transitionDuration: Duration(milliseconds: durationMs),
    transitionBuilder: getTransition(ani),
    //useRootNavigator: useRootNavigator,
  );
}

Widget Function(
    BuildContext context,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child) getTransition(NavAni ani) {
  switch (ani) {
    case NavAni.Left:
      return _buildFromLeftTransition;
    case NavAni.Right:
      return _buildFromRightTransition;
    case NavAni.Top:
      return _buildFromTopTransition;
    case NavAni.Bottom:
      return _buildFromBottomTransition;
    case NavAni.Blink:
    case NavAni.Fade:
      return _buildFromFadeTransition;
      break;
    case NavAni.Ripple:
      // TODO: Handle this case.
      break;
  }
}
